function [Vsolution, feasibility,power_beamform] = reweightedDC_SG(params,H_mat,V0)

%prob_to_socp: maps PARAMS into a struct of SOCP matrices
%input struct 'parms' has the following fields:
%params.N;   %'L': # RRHs
%params.K;    %'K': # MUs
%params.L_set;  %set of antennas at all the RRHs
%params.Active_number;   %number of active RRHs
%params.delta_set; %set of noise covariance

%%%%%%%%%%%%%%Problem Instances%%%%%%%%%%%%%
%params.r_set;  %set of SINR thresholds
%params.H;  %Channel Realization
%params.P_set;   %set of transmit power constraints at all the RRHs

%%%%%%%%Problem Data%%%%%%%
verb = params.verbosity;
K=params.K;   %Numbers of Mobile Users
r_set=params.r_set;     %Nx1 vector: QoS Requirements of Mobile Users for each Muliticast Group: assuming all the users have the same QoS requirments in the same group
L = params.L;
% delta_set=params.delta_set;  %Mx1 vector: noise covariance: assuming same value in the same group
ranktol = params.ranktol;
N=params.N;   %Lx1 vector: RAU antennas set
P_set=params.P_set;  %Nx1 vector: RAU transmit power set
S = params.S;
amcoeff=params.amcoeff; 
sigma_square = params.sigma_square;
Pc = params.Pc;
% activeset = params.activeset;
% weight=params.weight(activeset); %length(Active_index)x1: weigth for each group beamformer
weight=params.weight;
mu = params.weight;
%%%%%%%%CVX%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
LC = size(H_mat,4);


cvx_quiet(true)
cvx_begin sdp
variable V(K*N*L,1) complex;   %Variable for beamforming matrix, V_ij^l = v_il*vjl' = V(((i-1)*K+l-1)*L+1:((i-1)*K+l)*L,((j-1)*K+l-1)*L+1:((j-1)*K+l)*L);
my_obj=0;
    for n=1:N
        for k = 1:K
            my_obj = my_obj+(1/amcoeff+mu(k,n)*Pc)*sum_square_abs(V(((k-1)*N+n-1)*L+1:((k-1)*N+n)*L)); % power consumption
        end
    end
minimize (my_obj)  % group sparsity inducing minimization
subject to

%% RAUs Transmit Power Constraints
%% QoS Constraints
expression Qk(K,LC)
for lc = 1:LC
    for k=1:K
        gamma = r_set(k);
        for l =1:K
            if l==k
                eta = 1/gamma;
            else
                eta = -1;
            end
            tmp = 0;
            for n=1:N
                tmp=tmp+ vec(H_mat(n,:,k,lc))'*V(((l-1)*N+n-1)*L+1:((l-1)*N+n)*L)
            end
            Qk(k,lc) = Qk(k,lc)+eta*sum_square_abs(tmp);
        end
        Qk(k,lc)-sigma_square>=0;
    end
expression power_rau(N,1)
for n = 1:N
    for k=1:K
        power_rau(n) = power_rau(n)+sum_square_abs(V(((k-1)*N+n-1)*L+1:((k-1)*N+n)*L));
    end
    power_rau(n)<=P_set(n);
end

cvx_end
    
if strcmp(cvx_status,'Infeasible') || strcmp(cvx_status,'Failed')
    feasibility = 0;
    Vsolution = nan;     power_beamform = nan;
    return;
end
feasibility = 1;
Vsolution = V;       
power_beamform = zeros(K,N);
for n = 1:N
    for k = 1:K
        power_beamform(k,n) = V(((k-1)*N+n-1)*L+1:((k-1)*N+n)*L);
    end
end
end 
